 /*************************************************************************
Title........ calypso16.h
Date......... 14 November 2003
Author....... Alan Austin
Description..
*************************************************************************/

/****	Defines */

/* video sync period in ms */
#define	VSYNC_MS	16


/* Memory addressing */

/* video display memory: start address in CPU address space */
#define VIDEO_MEMORY_START (void*)0x3000000

/* video display memory: start address in CREMSON address space */
/* only top 16 Mbyte of 32 Mbyte memory is fitted */
#define CREMSON_MEMORY_START	((LONG)0x1000000)

/* display list defines */
/* start address in video memory*/

#define DISPLAY_LIST_START (LONG*)(0x3000000+0xf40000)
#define NLISTS	(WORD)(1<<2)	/* four display lists - do not change */
#define NLISTS_MASK	(WORD)(NLISTS-1)

/*
Size of each display list in entries.
entry size is long word  (4 bytes)

Calypso 16 has 256K memory reserved for display lists

Therefore: 4 x DISPLAY_LIST_SIZE x NLISTS <= 256x1024
*/
#define DISPLAY_LIST_SIZE (LONG)(16*1024)

/* maximum number of simultaneous animations */
#define MAX_ANIMATIONS	((WORD)8)

/* REGISTER addressing */

/* Pluto 5: CS3 Base Address  in CPU address space */
#define CS3			0x3000000

/* Cremson address */
#define Cremson_Host_Base		CS3+0xfc0000
#define Cremson_Display_Base	CS3+0xfd0000
#define Cremson_Draw_Base		CS3+0xff0000

/* Cremson Registers */

/* Host Interface Registers */
#define Cremson_DTC		Cremson_Host_Base+0x00	/* DMA Transfer Count Register  */
#define Cremson_DSU		Cremson_Host_Base+0x04	/* DMA Setup Register  */
#define Cremson_DRM		Cremson_Host_Base+0x05	/* DMA Request Mask Register  */
#define Cremson_DST		Cremson_Host_Base+0x06	/* DMA Status Register  */
#define Cremson_DTS		Cremson_Host_Base+0x08	/* DMA Transfer Stop Register  */
#define Cremson_LSTA	Cremson_Host_Base+0x10	/* Display List Transfer Status Register  */
#define Cremson_DRQ		Cremson_Host_Base+0x18	/* DMA Request Register  */
#define Cremson_IST		Cremson_Host_Base+0x20	/* Interrupt Status Register  */
#define Cremson_IMASK	Cremson_Host_Base+0x24	/* Interrupt Mask Register  */
#define Cremson_SRST	Cremson_Host_Base+0x2c	/* Software Reset Register  */
#define Cremson_LSA		Cremson_Host_Base+0x40	/* Display List  Source Address Register  */
#define Cremson_LCO		Cremson_Host_Base+0x44	/* Display List Count Register  */
#define Cremson_LREQ	Cremson_Host_Base+0x48	/* Display List Transfer Request Register  */

/* Graphics Memory Interface Registers */
#define Cremson_MMR		Cremson_Host_Base+0xfffc	/* Memory IF Mode Register  */

/* Display Control Registers */
#define Cremson_DCM		Cremson_Display_Base+0x00	/* Display Control Mode Register  */
#define Cremson_DCE		Cremson_Display_Base+0x02	/* Display Controller Enable Register  */
#define Cremson_HTP		Cremson_Display_Base+0x06	/* Hor Total Pixels Register  */
#define Cremson_HDP		Cremson_Display_Base+0x08	/* Hor Display Period Register  */
#define Cremson_HDB		Cremson_Display_Base+0x0a	/* Hor Display Boundary Register  */
#define Cremson_HSP		Cremson_Display_Base+0x0c	/* HSync Position Register  */
#define Cremson_HSW		Cremson_Display_Base+0x0e	/* HSync Width Register  */
#define Cremson_VSW		Cremson_Display_Base+0x0f	/* Vsync Width Register  */
#define Cremson_VTR		Cremson_Display_Base+0x12	/* Vert Total Rasters Register  */
#define Cremson_VSP		Cremson_Display_Base+0x14	/* Vsync Position Register  */
#define Cremson_VDP		Cremson_Display_Base+0x16	/* Vert Display Period Register  */
#define Cremson_WX		Cremson_Display_Base+0x18	/* Window Position X Register  */
#define Cremson_WY		Cremson_Display_Base+0x1a	/* Window Position Y Register  */
#define Cremson_WW		Cremson_Display_Base+0x1c	/* Window Width Register  */
#define Cremson_WH		Cremson_Display_Base+0x1e	/* Window Height Register  */
#define Cremson_CM		Cremson_Display_Base+0x20	/* C Layer Mode */
#define Cremson_COA		Cremson_Display_Base+0x24	/* C Layer Origin Address */
#define Cremson_CDA		Cremson_Display_Base+0x28	/* C Layer Display Address  */
#define Cremson_CDX		Cremson_Display_Base+0x2c	/* C Layer Display Position X */
#define Cremson_CDY		Cremson_Display_Base+0x2e	/* C Layer Display Position Y */
#define Cremson_WM		Cremson_Display_Base+0x30	/* W Layer Mode */
#define Cremson_WOA		Cremson_Display_Base+0x34	/* W Layer Origin Address */
#define Cremson_WDA		Cremson_Display_Base+0x38	/* W Layer Display Address */
#define Cremson_MLM		Cremson_Display_Base+0x40	/* ML Layer Mode				*/
#define Cremson_MLOA0	Cremson_Display_Base+0x44	/* ML Layer Origin Address 0	*/
#define Cremson_MLDA0	Cremson_Display_Base+0x48	/* ML Layer Display Address 0	*/
#define Cremson_MLOA1	Cremson_Display_Base+0x4c	/* ML Layer Origin Address 1	*/
#define Cremson_MLDA1	Cremson_Display_Base+0x50	/* ML Layer Display Address 1	*/
#define Cremson_MLDX	Cremson_Display_Base+0x54	/* ML Layer Display Position X	*/
#define Cremson_MLDY	Cremson_Display_Base+0x56	/* ML Layer Display Position Y	*/
#define Cremson_MRM		Cremson_Display_Base+0x58	/* MR Layer Mode				*/
#define Cremson_MROA0	Cremson_Display_Base+0x5c	/* MR Layer Origin Address 0	*/
#define Cremson_MRDA0	Cremson_Display_Base+0x60	/* MR Layer Display Address 0	*/
#define Cremson_MROA1	Cremson_Display_Base+0x64	/* MR Layer Origin Address 1	*/
#define Cremson_MRDA1	Cremson_Display_Base+0x68	/* MR Layer Display Address 1	*/
#define Cremson_MRDX	Cremson_Display_Base+0x6c	/* MR Layer Display Position X	*/
#define Cremson_MRDY	Cremson_Display_Base+0x6e	/* MR Layer Display Position Y	*/
#define Cremson_BLM		Cremson_Display_Base+0x70	/* BL Layer Mode				*/
#define Cremson_BLOA0	Cremson_Display_Base+0x74	/* BL Layer Origin Address 0	*/
#define Cremson_BLDA0	Cremson_Display_Base+0x78	/* BL Layer Display Address 0	*/
#define Cremson_BLOA1	Cremson_Display_Base+0x7c	/* BL Layer Origin Address 1	*/
#define Cremson_BLDA1	Cremson_Display_Base+0x80	/* BL Layer Display Address 1	*/
#define Cremson_BLDX	Cremson_Display_Base+0x84	/* BL Layer Display Position X	*/
#define Cremson_BLDY	Cremson_Display_Base+0x86	/* BL Layer Display Position Y	*/
#define Cremson_BRM		Cremson_Display_Base+0x88	/* BR Layer Mode				*/
#define Cremson_BROA0	Cremson_Display_Base+0x8c	/* BR Layer Origin Address 0	*/
#define Cremson_BRDA0	Cremson_Display_Base+0x90	/* BR Layer Display Address 0	*/
#define Cremson_BROA1	Cremson_Display_Base+0x94	/* BR Layer Origin Address 1	*/
#define Cremson_BRDA1	Cremson_Display_Base+0x98	/* BR Layer Display Address 1	*/
#define Cremson_BRDX	Cremson_Display_Base+0x9c	/* BR Layer Display Position X	*/
#define Cremson_BRDY	Cremson_Display_Base+0x9e	/* BR Layer Display Position Y	*/
#define Cremson_BRATIO	Cremson_Display_Base+0xb4	/* C Layer Blend Ratio Control */
#define Cremson_BMODE	Cremson_Display_Base+0xb6	/* C Layer Blend Mode Control */
#define Cremson_CTC		Cremson_Display_Base+0xbc	/* C Layer Transparent Control */
#define Cremson_MRTC	Cremson_Display_Base+0xc0	/* MR Layer Transparent Control */
#define Cremson_MLTC	Cremson_Display_Base+0xc2	/* ML Layer Transparent Control */


#define Cremson_CPal	Cremson_Display_Base+0x400	/* C Layer Palette Base */
#define Cremson_MBPal	Cremson_Display_Base+0x800	/* C Layer Palette Base */

/* Draw Control Registers */
#define Cremson_CTR		Cremson_Draw_Base+0x400		/* Control */

/* Draw Mode Parameter Registers accessed via SetRegister Display List Command */
#define Cremson_MDR0	0x420						/* Misc Mode Register */
#define Cremson_MDR4	0x430						/* BLT Mode Register */
#define Cremson_FBR		0x440						/* Drawing Frame Base Register */
#define Cremson_XRES	0x444						/* Drawing Frame Base Register */
#define Cremson_FC		0x480						/* Foreground Color */
#define Cremson_BC		0x484						/* Background Color */

/* Display List Commands */

#define SetRegister     0xf1
#define BltCopyAltP     0x0f
#define DrawBitmapP     0x0b
#define Interrupt		0xfd
#define Nop				0xff
#define DrawRectP		0x09

#define TopLeft         0x44
#define BltDraw         0x42
#define BitMap          0x43
#define BltFill         0x41

/* buffer modes*/
#define SINGLEBUFFER	0
#define DOUBLEBUFFER	1

/* layers */
#define C_LAYER			0x00
#define W_LAYER			0x01
#define ML_LAYER		0x02
#define MR_LAYER		0x03
#define BL_LAYER		0x04
#define BR_LAYER		0x05

/* layer selects - old style */
#define CONSOLE_ENABLE	0x01
#define WINDOW_ENABLE	0x02
#define MIDDLE_ENABLE	0x04
#define BASE_ENABLE		0x08
#define ALL_ENABLE		0x0f

/* layer selects - new style */
#define CONSOLE_SELECT	0x01
#define WINDOW_SELECT	0x02
#define MIDDLE_SELECT	0x04
#define BASE_SELECT		0x08
#define ALL_SELECT		0x0f

/* double buffer frames */
#define DOUBLEBUFFER_FRAME0		0
#define DOUBLEBUFFER_FRAME1		1
#define	DOUBLEBUFFER_AUTO		2

/* total number of display layers */
#define NLAYERS	6

/* palette select */
#define CONSOLE_LAYER_PALETTE		0
#define MIDDLE_BASE_LAYER_PALETTE	1


/* color modes */
#define INDIRECT_COLOR	0
#define DIRECT_COLOR	1


/* update sprite mode */
#define NORMAL_MODE 0
#define TRANSPARENT_MODE 0x08


/* BLT ROP function definitions */
#define BLT_CLEAR	((LONG)0x0100)
#define BLT_COPY	((LONG)0x0600)
#define BLT_AND		((LONG)0x0300)
#define BLT_OR		((LONG)0x0f00)


/* standard vga palette colours */
#define	 BLACK			0
#define	 BLUE			1
#define	 GREEN			2
#define	 CYAN			3
#define	 RED			4
#define	 MAGENTA		5
#define	 BROWN			6
#define	 LIGHTGRAY		7
#define	 DARKGRAY		8
#define	 LIGHTBLUE		9
#define	 LIGHTGREEN		10
#define	 LIGHTCYAN		11
#define	 LIGHTRED		12
#define	 LIGHTMAGENTA	13
#define	 YELLOW			14
#define	 WHITE			15

#define TRANSPARENT		255
#define	NORMAL			0


/****	Type Definitions */

/* palette entries */
typedef	struct palette_rgb {
	UBYTE	alpha;
	UBYTE	red;
	UBYTE	green;
	UBYTE	blue;
} PALETTE_RGB;


/* calypso config structure */
typedef	struct calypsocfg {
	const WORD* screenmode;
	const PALETTE_RGB*	palette;
} CALYPSOCFG;


/* display list */
typedef struct displaylist {
		LONG count;
		LONG* param;
} DISPLAYLIST;

typedef struct calypsolayer {
	LONG mode;				/* SINGLEBUFFER | DOUBLEBUFFER */
	BYTE color_mode;		/* INDIRECT_COLOR | DIRECT_COLOR 	8bit / 16bit */
	WORD transparent_color;
	WORD width;	    		/* layer width in pixels */
	WORD height;			/* layer height in lines */
	WORD xoffset;			/* offset from top left to start address: x coordinate in pixels  */
	WORD yoffset;			/* offset from top left to start address: y coordinate in lines   */
	LONG orig_adr0;			/* start address double buffer frame 0 */
	LONG orig_adr1;			/* start address double buffer frame 1 */
	LONG Xpos;				/* drawing position: x coordinate in pixels */
	LONG Ypos;				/* drawing position: y coordinate in lines  */
} CALYPSOLAYER;

/* vertical frame variables */
typedef	struct	framevars {
	BYTE framehz;		/* vertical frame rate - enumerated list below */
	BYTE framecount;	/* count up to framehz */
	BYTE frameflag;		/* frame flip flag */
	WORD framenumber;	/* increments every frame */
	BYTE db_frame;		/* currently displayed double buffer frame 0 or 1 */
	BYTE db_enable;		/* double buffering enable - if enabled flip at vertical frame rate */
} FRAMEVARS;

enum {FRAME_0HZ,FRAME_60HZ,FRAME_30HZ,FRAME_20HZ,FRAME_15HZ,FRAME_12HZ,FRAME_10HZ,FRAME_5HZ=12,FRAME_3HZ=20,FRAME_2HZ=30,FRAME_1HZ=60};


typedef struct calypsoimage {
	ULONG size_bytes;			/* image size bytes */
	WORD color8;				/* 1=8bit 0=16bit */
	WORD palette_size;			/* number of palette entries */
	WORD width;					/* width in pixels */
	WORD height;				/* height in lines */
	const PALETTE_RGB *palette;	/* palette data */
	const WORD *data;			/* image data */
} CALYPSOIMAGE;


/*** Animation control ***/

/* animation types */
enum {
	ANIMATION_TYPE1,	/* update position --> put sprite */
	ANIMATION_TYPE2,	/* clear area --> update position --> put sprite */
	ANIMATION_TYPE3		/* restore background --> update position --> save background -->put sprite */
};


/* animation run control bit fields */
enum {ANIMATION_STOP, ANIMATION_RUN, ANIMATION_PAUSE, ANIMATION_SINGLE_STEP};


#define ANIMATION_RUN_FOREVER		0


typedef struct animation {
	CALYPSOLAYER *layer;/* display layer */
	UWORD n_frames;		/* total number of frames in animation. ( nframes=0 - run forever ) */
	UWORD current_frame;/* current frame. Increments each frame. Animation stops when current_frame = n_frames */
	WORD start_x;		/* start position x */
	WORD start_y;		/* start position y */
	WORD start_dx;		/* start velocity x */
	WORD start_dy;		/* start velocity y */
	WORD x;				/* current position x */
	WORD y;				/* current position y */
	WORD dx;			/* current velocity x */
	WORD dy;			/* current velocity y */
	WORD save_x[2];		/* saved x [double buffer frame] (equals -1 if no value saved) */
	WORD save_y[2];		/* saved y [double buffer frame] */
	LONG image_start;	/* image: video memory start address */
	LONG image_offset;	/* multiple image sequence: video memory offset between consecutive images */
	WORD n_images;		/* multiple image sequence: number of images */
	WORD image_index;	/* multiple image sequence: current image index */
	LONG width;			/* image: width in pixels */
	LONG height;		/* image: height in lines */
	BYTE type;			/* ANIMATION_TYPE1, ANIMATION_TYPE2, ANIMATION_TYPE3 */
	LONG mode;			/* Blit mode: NORMAL_MODE or TRANSPARENT_MODE */
	BYTE run_control;	/* ANIMATION_STOP, ANIMATION_RUN, ANIMATION_PAUSE, ANIMATION_SINGLE_STEP */
	BYTE frame_advance;	/* If run_control=ANIMATION_SINGLE_STEP, advance to next frame when frame_advance true */
	BYTE state;			/* state variable */
	LONG save_buffer[2];/* save buffer addresses for saving background (type = ANIMATION_TYPE3) */
	void (*update)(struct animation* anim);
						/* pointer to user supplied update function - calculates next x,y */
} ANIMATION;

/* nodes for linked list used for animations */
typedef struct node {
	ANIMATION* anim;	/* each node defines a single animation */
	struct node *next;	/* pointer to next node */
} NODE;

/* animation linked list */
typedef struct animationlist {
	NODE *nodes;
	NODE *head;
} ANIMATIONLIST;


/* character fonts */

/* Unicode character handling: 512 font pages of 128 characters */
#define PAGE_SIZE	128
#define NPAGES	512

typedef struct calypsochar {
	UWORD offset;
	UBYTE dx;
	UBYTE dy;
	UBYTE width;
	UBYTE height;
	BYTE xoff;
	BYTE yoff;
} CALYPSOCHAR;

typedef struct calypsofont {
	UWORD code_page;
	UBYTE width;
	UBYTE height;
	BYTE xoff;
	BYTE yoff;
	CALYPSOCHAR char_metrics[PAGE_SIZE];
	const UBYTE *bitmaps;
} CALYPSOFONT;

typedef	struct calypsoservices {
	void  (*initdisplaymode)(const WORD *reg_table);
	void  (*videoon)();
	void  (*videooff)();
	UBYTE (*getvsync)();
	ULONG (*setvideomemory)(LONG video_addr, LONG size_bytes, BYTE color_mode, UWORD color);
	LONG  (*initlayer)(CALYPSOLAYER* layer, BYTE color_mode, BYTE buffer_mode, WORD transparent_color, WORD width_pixels, WORD height, LONG start_address);
	void  (*enablelayer)(UBYTE select);
	void  (*disablelayer)(UBYTE select);
	void  (*scrolllayerx)(CALYPSOLAYER* layer, ULONG offset);
	void  (*scrolllayery)(CALYPSOLAYER* layer, ULONG offset);
	WORD  (*clearlayer)(CALYPSOLAYER* layer, BYTE doublebuffer_frame);
	WORD  (*filllayer)(CALYPSOLAYER* layer, BYTE doublebuffer_frame, UWORD color, LONG mode);
	void  (*setwindowlayer)(WORD x_position, WORD y_position, WORD width, WORD height);
	void  (*setframerate)(BYTE rate);
	void  (*enabledoublebuffer)();
	void  (*disabledoublebuffer)();
	void  (*setdoublebufferframe)(BYTE frame);
	BYTE  (*getdoublebufferframe)();
	void  (*setframenumber)(WORD n);
	WORD  (*getframenumber)();
	void  (*frameadvance)();
	void  (*screenpartitionx)(WORD partition_xpos);
	void  (*alphablend)(BYTE enable);
	void  (*setblendratio)(BYTE blend_ratio, BYTE select);
	void  (*loadpalette)(UBYTE palette_select, const PALETTE_RGB* pal_table);
	void  (*getpalette)(UBYTE palette_select, UBYTE entry,PALETTE_RGB* col);
    void  (*setpalette)(UBYTE palette_select, UBYTE entry,PALETTE_RGB* col);
	LONG  (*loadimage)(LONG video_addr, const CALYPSOIMAGE* image);
	LONG  (*loadimagelayer)(CALYPSOLAYER* layer, BYTE doublebuffer_frame, const CALYPSOIMAGE* image);
	LONG  (*loadimageconvert8to16)(LONG video_addr, const CALYPSOIMAGE* image);
	LONG  (*loadimagelayerconvert8to16)(CALYPSOLAYER* layer, BYTE doublebuffer_frame, const CALYPSOIMAGE* image);
	LONG  (*loadsprite)(LONG video_addr, const CALYPSOIMAGE* image);
	WORD  (*putsprite)(CALYPSOLAYER* layer, BYTE doublebuffer_frame,LONG x,LONG y,LONG image_start,LONG xsize,LONG ysize,LONG mode);
	WORD  (*savesprite)(CALYPSOLAYER* layer, BYTE doublebuffer_frame,LONG x, LONG y, LONG save_address,LONG xsize,LONG ysize);
	WORD  (*clearsprite)(CALYPSOLAYER* layer, BYTE doublebuffer_frame, LONG x, LONG y, LONG xsize, LONG ysize);
	WORD  (*fillsprite)(CALYPSOLAYER* layer, BYTE doublebuffer_frame, LONG x, LONG y, LONG xsize, LONG ysize, UWORD color, LONG mode);
	void  (*setasciifont)(const CALYPSOFONT *font);
	void  (*setunicodefont)(const CALYPSOFONT *font, UWORD unicode_page);
	CALYPSOFONT*  (*getunicodefont)(UWORD unicode_page);
	WORD  (*textcolor)(UWORD foreground_color, UWORD background_color);
	void  (*settextposition)(CALYPSOLAYER* layer, WORD x,WORD y);
	WORD  (*gettextpositionx)(CALYPSOLAYER* layer);
	WORD  (*gettextpositiony)(CALYPSOLAYER* layer);
	WORD  (*vputchar)(CALYPSOLAYER* layer, BYTE doublebuffer_frame,UBYTE ch);
	WORD  (*vputstr)(CALYPSOLAYER* layer, BYTE doublebuffer_frame,BYTE *str);
	WORD  (*vputunicodechar)(CALYPSOLAYER* layer, BYTE doublebuffer_frame,UWORD ch);
	WORD  (*vputunicodestr)(CALYPSOLAYER* layer, BYTE doublebuffer_frame,UWORD *str);
	void  (*point)(CALYPSOLAYER* layer, LONG x, LONG y, UWORD color);
	void  (*line)(CALYPSOLAYER* layer, LONG Ax, LONG Ay, LONG Bx, LONG By, UWORD color);
	void  (*circle)(CALYPSOLAYER* layer, LONG xcentre, LONG ycentre, LONG radius, UBYTE color);
	void  (*defineanimation)(ANIMATION* anim, CALYPSOLAYER* layer, UWORD n_frames,
			WORD x, WORD y, WORD dx, WORD dy,
			LONG image_start, LONG image_offset, WORD n_images, WORD image_index,
			LONG width, LONG height,
			BYTE type, LONG mode,
			LONG buffer1, LONG buffer2,
			void (*update)(ANIMATION* anim));
	void  (*clearanimationlist)();
	WORD  (*addtoanimationlist)(ANIMATION* anim);
	WORD  (*removefromanimationlist)(ANIMATION* anim);
	void  (*startanimation)(ANIMATION* anim);
	void  (*stopanimation)(ANIMATION* anim);
	void  (*pauseanimation)(ANIMATION* anim);
	void  (*stepanimation)(ANIMATION* anim);
} CALYPSOSERVICES;


/****	Prototypes */

#include	"calypso16.p"

/*************************************************************************/
